/*
 * NQM_Regler.c
 *
 * Created: 09.08.2016 10:46:50
 *
 * Systemtakt 1000000Hz
 *
 * Master-Controller
 */ 

/*************** Bibliotheken ************************************************/

/*************** Systembibliotheken ******************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>         //noetig fr Zahlumwandlung mit itoa
#include <util/twi.h> 	    //enthaelt z.B. die Bezeichnungen fr die Statuscodes in TWSR
#include <stdint.h> 	    //definiert den Datentyp uint8_t

/*************** eigene Bibliotheken *****************************************/
#include "Variablen.h"
#include "General.h"
//#include "RS232.h"
#include "Delay.h"
#include "TWI_Master.h"
#include "Delay.h"

#include "TWI_Master.c"
#include "Delay.c"



/*
* Delay-Funktionen:
* Delay_ms(10);								// Funktionsaufruf im ms-Bereich
* Delay_us(100);							// Funktionsaufruf im s-Bereich
*/


int main(void)
{
	/* Peripherie einrichten und konfigurieren */
	/* Portdefinitionen */
	DDRB = 0xFF;							// als Ausgang maskiert
	PORTB = 0x00;							// zunaechst alle Ausgaenge
											// ausschalten
	DDRD = 0b00000010;						// Interrupteingaenge + USART
	PORTB = 0b00000000;						// Pull-Down einschalten

	GICR |= (1 << INT1)|(1 << INT0);		// externe Interrupts
											// einschalten
	MCUCR |= (1 << ISC10)|(1 << ISC11)|		// beide Interrupts auf
			(1 << ISC01)|(1 << ISC00);		// steigende Flanke
											// getriggert
											
//	RS232_Init ();
	
	if (!TWIM_Init (10000))
	{
//		printf ("Error in initiating TWI interface\n");
		while (1);
	}

	sei();									// Interrupts freigeben
	
//	uint8_t		byte1;

	
	
	
	Daten_holen = 1;						// fuer Start um die Daten
											// vom Slave zu holen, bevor
											// etwas anderes begonne wird
	
    while (1) 
    {
/******************* Kommunikation *******************************************/
/*

*/
// {
while(Daten_holen == 1)
{
	if (!TWIM_Start (SlaveAddress, TWIM_READ))
	{
		PORTB = 0b00010000;
		Delay_ms(300);
		PORTB = 0b00000000;
		Delay_ms(300);
		PORTB = 0b00010000;
		Delay_ms(300);
		PORTB = 0b00000000;
		TWIM_Stop ();
//		printf ("Could not start TWI Bus for READ\n");
	}
	else
	{
//		for (i=0;i<7;i++)
//		{
//			Data[i] = TWIM_ReadAck ();
//			printf ("Reading Byte %d: %d\n", i, Data[i]);
//		}
		Data[0] = TWIM_ReadNack ();	
//		Data[1] = TWIM_ReadNack();
//		printf ("Reading Byte %d: %d\n", i, Data[7]);
		TWIM_Stop ();
//		Delay_ms (1000);
		starten = 1;
		Daten_holen = 0;
	}
	
//	sei();										// alle Interrupts wieder
												// freigeben, da fertig
	

}
// }



while (starten == 1)
{
	/******************* Funktionen **********************************************/
	/*
	* Die Funktionen sind anhand der vergebenen Namen selbsterklaerend.
	*/
	// {
	/******************* Berechnungen ********************************************/
	/*
	* Zunaechst wird der ermittelte Wert vom ADC umgerechnet in den realen
	* physikalischen Wert der Spannung. Anschlieend wird das Toleranzband
	* berechnet.
	* Der reale Spannungswert entspricht dem Effektivwert nach der Umrechnung.
	* Formel digitaler Wert in analogen Wert:
	* analoger Wert = (Byteinhalt / Aufloesung) * Referenzspannung
	* analoger Wert = (Byteinhalt / 255) * 5V
	* Formel Umrechnung in Effektivwert:
	* Effektivwert = (analoger Wert * Verhltnis) / Wurzel(2)
	*/
		// {
	maximaler_spannungswert = Data[0];

	// Der reale Spannungswert wird nur fuer die Anzeige auf einem Display bentigt.
	relaer_Spannungswert = 
	(((maximaler_spannungswert / 255) * 5) * Verhaeltnis) / Wurzel2;

	// Muss hier stehen, da Kunde Werte fuer Toleranzen aendern kann.
	// Kann also nicht nur einmalig ausgefuehrt werden, sondern muss mit jedem
	// Eintreten neu berechnet werden.
	Toleranzgrenze_unten_berech = gewuenschter_Spannungswert - 
									((gewuenschter_Spannungswert * Toleranz_unten)
																		 / 100);
	Toleranzgrenze_unten = Toleranzgrenze_unten_berech;
	Toleranzgrenze_oben_berech = gewuenschter_Spannungswert + 
									((gewuenschter_Spannungswert * Toleranz_oben) 
																		/ 100);
	Toleranzgrenze_oben = Toleranzgrenze_oben_berech;
		// }


	/******************* "Regelung" **********************************************/
//	 {
	if (maximaler_spannungswert > Toleranzgrenze_oben)
	{
		Ueberschreitungen++;		// Laufvariable fuer Ueberschreitungen der
									// Ueberspannung hochzaehlen
	}
	else
	{
		Ueberschreitungen = 0;		// Zaehlen wird zurueck gesetzt, wenn Spannung
									// wieder im Toleranzband liegt, bevor geregelt
									// wird.
	}

	if (maximaler_spannungswert < Toleranzgrenze_unten)
	{
		Unterschreitungen++;		// Laufvariable fuer Unterschreitungen der
									// Unterspannung hochzaehlen
	}
	else
	{
		Unterschreitungen = 0;		// Zaehlen wird zurueck gesetzt, wenn Spannung
									// wieder im Toleranzband liegt, bevor geregelt
									// wird.
	}

	// Hier wird ueberprft ob die Ueberschreitungen zu oft waren
	if (Ueberschreitungen > Band)
	{
		Ausregeln_rechts = 1;		// wenn ja, wird Regelung rechts freigegeben
	}
	else
	{
		Ausregeln_rechts = 0;
	}

	// Hier wird ueberprft ob die Unterschreitungen zu oft waren
	if (Unterschreitungen > Band)
	{
		Ausregeln_links = 1;		// wenn ja, wird Regelung links freigegeben
	}
	else
	{
		Ausregeln_links = 0;
	}


	// Motor Rechtslauf regeln
	// Nur einmal durchlaufen, sonst unendlich regeln,
	// da kein neuer Wert geholt wird
	if ((Ausregeln_rechts == 1) && (Ausregeln_rechts_zu_weit != 0))
	{
		
/*		
		if (maximaler_spannungswert < Toleranzgrenze_oben)
		{
			
			Ausregeln_rechts = 2;		// Wenn Spannung wieder im Toleranzband
										// Regelung ausschalten
			Ueberschreitungen = 0;		// wieder zuruecksetzen
		}
		else
		{
*/
			if (Ausregeln_links_zu_weit == 0)
			{
				// erfolgt hier, weil der Regler dann wirklich von der Position
				// wegfaehrt
				PORTB = 0b00000000;		// rote LED ausschalten
				Ausregeln_links_zu_weit = 1;	
										// Sollte ein Linksdrehen an Endanschlag
										// erfolgt sein, wird es bei der
										// Gegendrehung zurueck gesetzt
			}
			/* Das Relais wird getoggelt und der Wert schrittweise ausgeregelt */
			PORTB = 0b00001000;			// Relais rechts drehen einschalten
			Delay_ms(delay_rechts);		// warten
										// Eine Richtung muss laenger aktiv
										// als die andere Richtung, falls
										// ueberregelt wird, waere es sonst
										// ein hin und her springen. Ist
										// ein Wert groeser, kann das 
										// Toleranzband wieder gefunden
										// werden.
			PORTB = 0b00000000;			// Relais rechts drehen ausschalten
			Delay_ms(delay_rechts);		// warten
			Ausregeln_rechts = 0;
//		} gehoert noch zur else-Klammer
	}

	// Motor Linkslauf regeln
	// Nur einmal durchlaufen, sonst unendlich regeln,
	// da kein neuer Wert geholt wird
	if ((Ausregeln_links == 1) && (Ausregeln_links_zu_weit != 0))
	{
/*
		if (maximaler_spannungswert > Toleranzgrenze_unten)
		{
			
			Ausregeln_links = 2;		// Wenn Spannung wieder im Toleranzband
										// Regelung ausschalten
			Unterschreitungen = 0;		// wieder zuruecksetzen
		}
		else
		{
*/
			// erfolgt hier, weil der Regler dann wirklich von der Position
			// wegfaehrt
			if (Ausregeln_rechts_zu_weit == 0)
			{
				PORTB = 0b00000000;		// rote LED ausschalten
				Ausregeln_rechts_zu_weit = 1;	
										// Sollte ein Rechtsdrehen an Endanschlag
										// erfolgt sein, wird es bei der
										// Gegendrehung zurueck gesetzt
			}
			/* Das Relais wird getoggelt und der Wert schrittweise ausgeregelt */
			PORTB = 0b00000100;			// Relais links drehen einschalten
			Delay_ms(delay_links);		// warten
			PORTB = 0b00000000;			// Relais links drehen ausschalten
			Delay_ms(delay_links);		// warten
			Ausregeln_links = 0;
//		} gehoert noch zur else-Klammer
	}
//	}
	Daten_holen = 1;
	starten = 0;
}


/******************* rote LED ************************************************/
// {
if ((Ausregeln_rechts_zu_weit == 0) || (Ausregeln_links_zu_weit == 0))
{
	PORTB = 0b00010000;				// rote LED einschalten
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
// Geht das so? Weil realer Spannungswert ist ein float.
/******************* Display *************************************************/
//uart_puti(relaer_Spannungswert);





    }
}
//	}
// }



/******************* Interrupts **********************************************/
// {
/******************* externe Interrupts **************************************/
/*
* Die beiden externen Interrupts werden ausgelst, wenn der Motor den
* jeweilgen Endanschlag erreicht. Der Motor wird dann in diese Richtung
* gesperrt. Die rote LED wird eingeschalten um anzuzeigen, dass ein Endanschlag
* erreicht wurden ist. Jedoch wird das Gert nicht aus der Funktion gerissen.
* Es wird gewartet bis alle Bedingngen erfllt sind, um in die Gegenrichtung zu
* fahren. Dann wird die rote LED wieder ausgeschalten.
*/

//		{
/* Motor STOPP Rechtslauf */
ISR (INT1_vect)
{
	Ausregeln_rechts_zu_weit = 0;
}
/* Motor STOPP Linkslauf */
ISR (INT0_vect)
{
	Ausregeln_links_zu_weit = 0;
}
//		}